%=============================================================================
% Filename: genlik_US.m
%
% Wavelet domain image denoising method described in
% "A Versatile Wavelet Domain Noise Filtration Technique for Medical Imaging",
% by A. Pizurica, W. Philips, I . Lemahieu and M. Acheroy, 2002.
%
% Spatial adaptation is achieved using a local spatial activity indicator 
% that is computed from a squared window. 
%
% The multiplication factor for the threshold that specifies the
% signal of interest is user-defined. The optimum value of this 
% parameter depends on the noise type. For non-correlated noise, 
% the optimum value is usually in the interval [1-2]. 
% For spatially correlated speckle noise, the optimum value is ~3.
% Larger values lead to a stronger smoothing of noise and weak image textures.
%
% This program allows 
% 1) denoising arbitrary input images 
% 2) generation of test images with artificial spatially correlated speckle
%    and denoising the generated test image
%
% In cases where the reference noise-free image is provided
% the program calculates the input and the resulting signal to noise ratio's
%
% Author: Aleksandra Pizurica <Aleksandra.Pizurica@telin.UGent.be>
%         TELIN, Ghent University
%
% Last modified: 7. May 2004.
%=============================================================================

clear all;
close all;
global InputType SCAN filename map;


Ref=input('Reference noise-free image available? (y/n) ','s');

% Read reference, noise-free image
if (Ref=='y')
    InputType=0;
    while (InputType~=1 & InputType~=2 & InputType~=3 & InputType~=4)  
     InputType=input('\nReference image format:\n1. "*.tif"\n2. "*.sg2"\n3. "*.mat"\n4. "*.bin"\n>>  ');
    end
    imload;
    Xclean=SCAN;
    clear SCAN;
    [M,N]=size(Xclean);
  
    Choice=input('Choose 1 or 2:\n1. An arbitrary noise variance\n2. A stored noisy image\n ');

    if(Choice==1)
       NoiseStd=input('Standard Deviation of speckle noise (0 to 1): ');
       Xnoisy=SimulateSpeckleFun(Xclean,NoiseStd);
       clean_image=input('Reference image name: ','s');
       image_name=strcat(clean_image,'_std',num2str(NoiseStd));
    else
       InputType=0;
       while (InputType~=1 & InputType~=2 & InputType~=3 & InputType~=4)  
         InputType=input('\nNoisy image format:\n1. "*.tif"\n2. "*.sg2"\n3. "*.mat"\n4. "*.bin"\n>>  ');
       end
       imload;
       Xnoisy=SCAN;
       clear SCAN;
  
       %Extract the image name from the filename
       image_name=filename(1:(length(filename)-4))
    end
  
else % No reference noise-free image image available

    % Read noisy image
    InputType=0;
    while (InputType~=1 & InputType~=2 & InputType~=3 & InputType~=4)  
        InputType=input('\nNoisy image format:\n1. "*.tif"\n2. "*.sg2"\n3. "*.mat"\n4. "*.bin"\n>>  ');
    end
    imload;
    Xnoisy=SCAN;
    clear SCAN;
    
    %Extract the image name from the filename
    image_name=filename(1:(length(filename)-4))
end

figure('units','normalized','position',[0.05 0.1 0.3 0.8]);
subplot(2,1,1),imagesc(Xnoisy);title('Input image');
colormap(gray);
 
[M,N]=size(Xnoisy); 
Mask_method=1;
 
 %--------------------------------------------------------------
 %Parameters

 Thr_factor=1;
 Thr_factorINFO='s';
 while((Thr_factorINFO~='y')&(Thr_factorINFO~='n'))
    Thr_factorINFO=input('The threshold multiplication factor is set to default value K=1. \nChoose other value (y/n)? ','s');
 end
 if(Thr_factorINFO=='y')
   Thr_factor=input('Multiplication factor for threshold (1-10): ');
 end
 
 WS=5;
 WS_INFO='s';
 while((WS_INFO~='y')&(WS_INFO~='n'))
   WS_INFO=input('Window size is set to default 5. \nChoose other value (y/n)? ','s');
 end
 if(WS_INFO=='y')
   WS=1;
   while((WS~=3)&(WS~=5)&(WS~=7))
      WS=input('Window size (3, 5, 7): ');
   end
end
W=(WS-1)/2;

 %-------------------------------------------------------------
 
 Norm=2550/(max(max(Xnoisy))); 
    
   [A1,HL1,LH1,HH1]=wt3det_spline(Norm*Xnoisy,0);   
   [A2,HL2,LH2,HH2]=wt3det_spline(A1,1);   
   [A3,HL3,LH3,HH3]=wt3det_spline(A2,2);  
   [A4,HL4,LH4,HH4]=wt3det_spline(A3,3);  
   [A5,HL5,LH5,HH5]=wt3det_spline(A4,4);  

  
  sigma=Thr_univ(HH1);
  if(sigma==0)
     sigma=max(Thr_univ(HL1),Thr_univ(LH1))/0.79;
  end
   SIGMA_HH=[1.00 0.22 0.09 0.04]*sigma;
   SIGMA_LH=[0.79 0.25 0.11 0.05]*sigma;
   
   sig_hh1=SIGMA_HH(1);
   sig_hh2=SIGMA_HH(2);
   sig_hh3=SIGMA_HH(3);
   sig_hh4=SIGMA_HH(4);

   sig_lh1=SIGMA_LH(1);
   sig_lh2=SIGMA_LH(2);
   sig_lh3=SIGMA_LH(3);
   sig_lh4=SIGMA_LH(4);



%====================================
% Processing scale 2^4

%Scale=4


%[HL4p,M_hl4]=rem_noise_adapt(HL4,HL5,sig_lh4,Thr_factor,W);
%[LH4p,M_lh4]=rem_noise_adapt(LH4,LH5,sig_lh4,Thr_factor,W);
%[HH4p,M_hh4]=rem_noise_adapt(HH4,HH5,sig_hh4,Thr_factor,W);

%figure
%subplot(3,2,1),imagesc(HL4);
%subplot(3,2,3),imagesc(LH4);
%subplot(3,2,5),imagesc(HH4);
%colormap(gray);

%subplot(3,2,2),imagesc(HL4p);
%subplot(3,2,4),imagesc(LH4p);
%subplot(3,2,6),imagesc(HH4p);
%colormap(gray);
%clear HL4 LH4 HH4 z;


%====================================
% Processing scale 2^3

Scale=3


[HL3p,M_hl3]=rem_noise_adapt(HL3,HL4,sig_lh3,Thr_factor,W);
[LH3p,M_lh3]=rem_noise_adapt(LH3,LH4,sig_lh3,Thr_factor,W);
[HH3p,M_hh3]=rem_noise_adapt(HH3,HH4,sig_hh3,Thr_factor,W);

%figure
%subplot(3,2,1),imagesc(HL3);
%subplot(3,2,3),imagesc(LH3);
%subplot(3,2,5),imagesc(HH3);
%colormap(gray);

%subplot(3,2,2),imagesc(HL3p);
%subplot(3,2,4),imagesc(LH3p);
%subplot(3,2,6),imagesc(HH3p);
%colormap(gray);
clear HL4 LH4 HH4 z;
%====================================
% Processing scale 2^2


[HL2p,M_hl2]=rem_noise_adapt(HL2,HL3,sig_lh2,Thr_factor,W);
[LH2p,M_lh2]=rem_noise_adapt(LH2,LH3,sig_lh2,Thr_factor,W);
[HH2p,M_hh2]=rem_noise_adapt(HH2,HH3,sig_hh2,Thr_factor,W);

Scale=2
%figure
%subplot(3,2,1),imagesc(HL2);
%subplot(3,2,3),imagesc(LH2);
%subplot(3,2,5),imagesc(HH2);
%colormap(gray);

%subplot(3,2,2),imagesc(HL2p);
%subplot(3,2,4),imagesc(LH2p);
%subplot(3,2,6),imagesc(HH2p);
%colormap(gray);

clear HL3 LH3 HH3;
%====================================
% Processing scale 2^1

Scale=1

[HL1p,M_hl1]=rem_noise_adapt(HL1,HL2p,sig_lh1,Thr_factor,W);
[LH1p,M_lh1]=rem_noise_adapt(LH1,LH2p,sig_lh1,Thr_factor,W);
[HH1p,M_hh1]=rem_noise_adapt(HH1,HH2p,sig_hh1,Thr_factor,W);
%figure
%subplot(3,2,1),imagesc(HL1);
%subplot(3,2,3),imagesc(LH1);
%subplot(3,2,5),imagesc(HH1);
%colormap(gray);

%subplot(3,2,2),imagesc(HL1p);
%subplot(3,2,4),imagesc(LH1p);
%subplot(3,2,6),imagesc(HH1p);
%colormap(gray);

clear LH2 HL2 HH2 LH1 HL1 HH1;

%=========================================
% Reconstruction
%=========================================

   %figure;
   %REC='y';
   %while(REC=='y')
   
   %REC_scales=input('Reconstruct from 1, 2, 3 or 4 modified scales? ');
   REC_scales=2;
   
          if(REC_scales==4)
             A3p=iwt3det_spline(A4,HL4p,LH4p,HH4p,3);
             A2p=iwt3det_spline(A3p,HL3p,LH3p,HH3p,2);
             A1p=iwt3det_spline(A2p,HL2p,LH2p,HH2p,1);
             A0p=iwt3det_spline(A1p,HL1p,LH1p,HH1p,0);
          elseif(REC_scales==3)
             A2p=iwt3det_spline(A3,HL3p,LH3p,HH3p,2);
             A1p=iwt3det_spline(A2p,HL2p,LH2p,HH2p,1);
             A0p=iwt3det_spline(A1p,HL1p,LH1p,HH1p,0);
          elseif(REC_scales==2)
             A1p=iwt3det_spline(A2,HL2p,LH2p,HH2p,1);
             A0p=iwt3det_spline(A1p,HL1p,LH1p,HH1p,0);
          else
             A0p=iwt3det_spline(A1,HL1p,LH1p,HH1p,0);
          end
  
   
Xden=A0p/(Norm);
  
clear A3 A4 

if(Ref=='y')
  Ps=std2(Xclean)^2;
  Pn=(std2(Xclean-Xnoisy))^2;
  Prn=(std2(Xclean-Xden))^2;
  SNRin=10*log10(Ps/Pn)
  SNRres=10*log10(Ps/Prn)  
  
  MSEres=mean(mean((Xclean-Xden).^2));
  MSEin=mean(mean((Xclean-Xnoisy).^2));

  PSNRres=10*log10(255^2/MSEres)
  PSNRin=10*log10(255^2/MSEin);
end

subplot(2,1,2),imagesc(Xden);
colormap(gray);
title('Denoised image');
   
%============================================
% Save result

Save_image_mat=input('Save the result as a mat file? y/n ','s')

if (Save_image_mat=='y');
   Filename=strcat(image_name,'_linfit_T',num2str(Thr_factor),'_W',num2str(WS),'.mat');
   X=Xden;
   WindowSize=WS;
   if(Ref=='y')
      save(Filename,'X','SNRin','SNRres','PSNRin','PSNRres','WindowSize','Thr_factor');
   else
      save(Filename,'X','WindowSize','Thr_factor');
   end 
end

Save_image_tif=input('Save the result as tif image? y/n ','s')

if (Save_image_tif=='y');
   Filename=strcat(image_name,'_linfit_T',num2str(Thr_factor),'_W',num2str(WS),'.tif');
   X=Xden;
   X(X<0)=0;
   X(X>255)=255;
   imwrite(uint8(X),gray(256),Filename,'tif');
end