%------------------------------------------------------------------------------------
% Function: rem_noise_adapt_roi.m
% 
% Remove noise from a given wavelet subband using the method of [1].
%
% This version allows the parameter estimation from a specified region of interest 
% (roi) from the image !!!!!!!!!!!
% 
% Input: 
%
%       D..........- input wavelet subband (detail image) 
%       Dcoars.....- the corresponding detail image at the next coarser scale
%       sigma......- noise standard deviation in the subband D
%       Thr_factor - the threshold multiplication factor for the signal of interest
%       W..........- parameter which describes the window size: w=(window size-1)/2
%       ROI....... - region from which the signal and noise statistics are estimated
% 
% Output: 
%       D_cl........- denoised subband D
%       Mask........- the estimated edge mask for the subband D
%
%[1] A. Pizurica, W. Philips, I. Lemahieu and M. Acheroy, 
%   "A Versatile Wavelet Domain Noise Filtration Technique for Medical Imaging," 
%    IEEE, Trans. on Medical Imaging, vol. 22, no. 3, pp. 323--331, March 2003. %
%
%  
% Author: Aleksandra Pizurica <Aleksandra.Pizurica@telin.UGent.be> 
%         TELIN/Ghent University
%
% Last modified: 7. May 2004.
%------------------------------------------------------------------------------------

function [D_cl,Mask]=rem_noise_adapt_roi(D,Dcoars,sigma,Thr_factor,W,ROI);

[M,N]=size(D);

Mask=init_mask(D.*Dcoars,(Thr_factor*sigma)^2);

O=sum(sum(Mask));
if(O<M*N)
   r=O/(M*N-O);
else
   r=100;
end

if(r<4e-4)
   r=0;
end

if (r==0)
   D_cl=zeros(M,N);
else
   [arg,likel_ratio]=approx_likel_ratio_roi(Mask,D,ROI);
   [arg,prior_ratio]=approx_prior_ratio_roi(Mask,D,Dcoars,W,ROI);
   SHRINK=adapt_energy(D,Dcoars,likel_ratio,prior_ratio,W,r);
   D_cl=D.*SHRINK;
end



   
